/* This file contains only functions related to sending messages over the
    ircd socket */
#include "options.h"
#include "aClient.h"
#include "player.h"
#include "item.h"
#include "pouch.h"
#include "extern.h"
#include "sockhelp.h"
#include "flags.h"

#include <fstream>

using namespace std;

void raw(const char *fmt, ...)
{
    va_list args;
    char *input, *buffer;
    const char *t = fmt;
    input = new char[2048];
    buffer = new char[1024];
    va_start(args, fmt);
    vsnprintf(buffer, 1024, fmt, args);
    va_end(args);

    memset(input, 0, sizeof(input)); // Initialize to NULL
    for (t = buffer; *t; t++)
    {
	if (*t == '<')
	{
	    switch(*++t) {
            case 'S': sprintf(input, "%s%s", input, s_GameServ); break;
	    }
	}
	else
	{
	    sprintf(input, "%s%c", input, *t);
	}

    }
    #ifdef DEBUGMODE
	log("Input: %s", input);
    #endif

    sprintf(input, "%s%s", input, "\r\n");
    sock_puts(sock, input);
    delete [] input;
    delete [] buffer;
}
/* Send a NOTICE from the given source to the given nick. */

void notice(const char *source, const char *dest, const char *fmt, ...)
{
    if (fmt[0] == '\0')
        return;

    char *commanduse;
    commanduse = new char[16];

    #ifdef P10
        if (isUsePrivmsg())
            strcpy(commanduse, "P");
        else
            strcpy(commanduse, "O");
    #else

        if (isUsePrivmsg())
            strcpy(commanduse, "PRIVMSG");
        else
            strcpy(commanduse, "NOTICE");
    #endif

    va_list args;
    char *input, *buffer;
    const char *t = fmt;
    input = new char[2048];
    buffer = new char[1024];
    va_start(args, fmt);
    vsnprintf(buffer, 1024, fmt, args);
    va_end(args);

    if (dest[0] == ':')
    {
        dest++;

      #if !defined(P10)
        sprintf(input, ":%s %s %s :", source, commanduse, dest);
      #else
        sprintf(input, "%s %s %s :", gsnum, commanduse, dest);
      #endif

        dest--;
    }
    else
    {
      #if !defined(P10)
        sprintf(input, ":%s %s %s :", source, commanduse, dest);
      #else
        sprintf(input, "%s %s %s :", gsnum, commanduse, dest);
      #endif
    }
    log("buffer: %s", buffer);
    for (t = buffer; *t; t++)
    {
        if (*t == '<')
        {
            if(*(++t) == 'S')
            {
             sprintf(input, "%s%s", input, s_GameServ);
            }
        }
        else
        {
            sprintf(input, "%s%c", input, *t);
        }

    }
    #ifdef DEBUGMODE
        log("Input: %s", input);
    #endif
    sprintf(input, "%s%s", input, "\r\n");
    sock_puts(sock, input);
    delete [] commanduse;
    delete [] input;
    delete [] buffer;
}

void showstats(const char *u, const char *nick)
{
    aClient *sender = find(u);
	Player *p;
    char *buf;
    buf = new char[50];
    string space;


    if (!(p = findplayer(nick)))
	  {
        notice(s_GameServ, u, "%s not found", nick);
	  }
    else
	  {
        notice(s_GameServ, sender->getNick(), "Stats for %s:", p->getName().c_str());
		
        sprintf(buf, "Experience: %ld", p->getExp());
        space = spaces(strlen(buf), " ");
        notice(s_GameServ, sender->getNick(), "%s%sLevel: %d",  buf, space.c_str(),
			   p->getLevel());
        
		
        sprintf(buf, "Gold: %ld", p->getGold());
        space = spaces(strlen(buf), " ");
        notice(s_GameServ, sender->getNick(), "%s%sGold in Bank: %ld", buf, space.c_str(), p->getBank());
        
		
        notice(s_GameServ, sender->getNick(), "Hit Points: %d of %d", p->getHP(),
			   p->getMaxHP());
		
        sprintf(buf, "Strength: %d", p->getStrength());
        space = spaces(strlen(buf), " ");
        notice(s_GameServ, sender->getNick(), "%s%sDefense: %d",
			   buf, space.c_str(), p->getDefense());
        
		
        sprintf(buf, "Armor: %s", (p->getArmor() ? p->getArmor()->getName().c_str() : "Nothing"));
        space = spaces(strlen(buf), " ");
		
        notice(s_GameServ, sender->getNick(), "%s%sWeapon: %s", buf, space.c_str(),
			   (p->getWeapon() ? p->getWeapon()->getName().c_str() : "Fists"));
		
        
		
        sprintf(buf, "Forest Fights: %d", p->getForestFights());
        space = spaces(strlen(buf), " ");
        notice(s_GameServ, sender->getNick(), "%s%sPlayer Fights: %d", buf, space.c_str(), p->getPlayerFights());
        
		if (p->getClient() == sender || isAdmin(sender))
		  {
			showinventory(p, sender);
		  }
	  }

    delete [] buf;
}

void display_monster(char *u)
{
  if (is_playing(u))
    {
	  aClient *user = find(u);
	  Player *ni = user->stats;
	  
	  notice(s_GameServ, u, "Your Hitpoints: %d", ni->getHP());
	  notice(s_GameServ, u, "%s's Hitpoints: %d", ni->getMonster()->name.c_str(), ni->getMonster()->hp);
	  notice(s_GameServ, u, "Here are your commands:");
	  notice(s_GameServ, u, "/msg <S attack");
	  notice(s_GameServ, u, "/msg <S run");
	  notice(s_GameServ, u, "What will you do?");
    }
}

void display_players(char *u)
{
  aClient *user;
  if (!(user = find(u)))
    {
	  log("Fatal error in display_players(): Couldn't find %s", u);
    }
  else
	display_players(user);
}

void display_players(aClient *user)
{
  char *u = user->getNick();
  if (is_playing(user) && player_fight(user))
    {
	  aClient *battle = user->stats->getBattle();
	  notice(s_GameServ, u, "Your Hitpoints: %d", user->stats->getHP());
	  notice(s_GameServ, u, "%s's Hitpoints: %d", battle->stats->getName().c_str(), battle->stats->getHP());
	  notice(s_GameServ, u, "Here are your commands:");
	  notice(s_GameServ, u, "/msg <S attack");
	  notice(s_GameServ, u, "/msg <S run");
	  notice(s_GameServ, u, "What will you do?");
    }
}

void showinventory(Player *from, aClient *to)
{
  char *nick;
  if (!to || !from)
	{
	  return;
	}

  nick = to->getNick();
  
  if (from->inventory->isEmpty())
	{
	  notice(s_GameServ, nick, "You aren't carrying anything");
	  return;
	}
  
  list <itemContainer> *items;
  items = from->inventory->getItems();
  
  list <itemContainer>::iterator item_iter, item_iter2;
  item_iter = items->begin();
  
  notice(s_GameServ, nick, "Inventory for %s:", from->getName().c_str());
  long count;
  while (item_iter != items->end())
	{
	  // Display a list of counted items so as not to show an extra line for duplicate items
	  count = 0;
	  item_iter2 = item_iter;
	  while (item_iter != items->end() && (*item_iter) == (*item_iter2))
	    {
	      if ((*item_iter) == (*item_iter2))
			{
			  ++count;
			}
	      else
			{
			  break;
			}
  	      ++item_iter;
	    }
	  notice(s_GameServ, nick, "%ld.) %s (%ld)", (*item_iter2).getItem()->getID(), (*item_iter2).getItem()->getName().c_str(), count);
	}
}


void showTavern(aClient *user)
{
  Player *p;
  list<tavernItem>::iterator item_iterator;
  item *tempItem;
  
  p = user->stats;
  
  if (!p)
    return;
  
  item_iterator = tavern.begin();
  if (tavern.empty())
    {
      notice(s_GameServ, user->getNick(), "Tavern is empty");
      return;
    }
  
  while (item_iterator != tavern.end())
    {
      if (p->getLevel() < (*item_iterator).getLevel())
		{
		  item_iterator++;
		  continue;
		}
      tempItem = (*item_iterator).getItem();
      notice(s_GameServ, user->getNick(), "%d. %s for %ld gold", tempItem->getID(), tempItem->getName().c_str(), tempItem->price());
      item_iterator++;
    }
}

void showBankBalance(const char *u)
{
  aClient *user;
  Player *p;
  
  if (!(user = find(u)))
	return;
  
  p = user->stats;
  
  if (!p)
	return;
  
  notice(s_GameServ, u, "Account Balance: %ld     Gold On hand: %ld", p->getBank(), p->getGold());
}

void display_help(const string &u, const string &file)
{
  ifstream infile;
  char *buf;
  
  if (file.empty())
    {
	  infile.open("helpfiles/help");
	  if (infile.fail())
		{
		  log("Error opening helpfiles/help");
		  notice(s_GameServ, u.c_str(), "Error opening helpfiles/help");
		  return;
		}
	  buf = new char[1024];
	  while(infile.getline(buf, 1024))
		{
		  // Written this way, it will process <S in the helpfiles
		  // Instead of notice(s_GameServ, u, "%s", buf);
		  notice(s_GameServ, u.c_str(), buf);
		}
	  
	  // Minor recursion - as long as admin_commands exists, it won't loop
          // continuously
	  aClient *user = find(u.c_str());
	  if (user && isAdmin(user))
	    display_help(u.c_str(), "admin_commands");
    }
  else
    {
	  string filename;
      filename = "helpfiles/";
      filename += file;
	  
	  for (unsigned int x = 10; x < filename.length(); x++)
	    filename[x] = tolower(filename[x]);
	  
	  infile.open(filename.c_str());
	  if (infile.fail())
		{
		  notice(s_GameServ, u.c_str(), "No help for %s", file.c_str());
		  return;
		}
	  buf = new char[1024];
	  while(infile.getline(buf, 1024))
		{
		  // Written this way, it will process <S in the helpfiles
		  // Instead of notice(s_GameServ, u, "%s", buf);
		  notice(s_GameServ, u.c_str(), buf);
		}
    }
  infile.close();
  delete [] buf;
}

